package com.tencent.igame.tip_flutter_image_view;

import android.content.Context;
import android.graphics.SurfaceTexture;
import android.view.SurfaceView;

import java.util.Map;

import androidx.collection.LongSparseArray;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;
import io.flutter.view.TextureRegistry;

/**
 * TipFlutterImageViewPlugin
 */
public class TipFlutterImageViewPlugin implements MethodCallHandler {
    /**
     * Plugin registration.
     */

    private final TextureRegistry textures;
    private Context context;

    private LongSparseArray<OpenGLRenderer> renders = new LongSparseArray<>();

    public TipFlutterImageViewPlugin(TextureRegistry textures, Context context) {
        this.textures = textures;
        this.context = context;
    }

    public static void registerWith(Registrar registrar) {
        final MethodChannel channel = new MethodChannel(registrar.messenger(), "com.tencent.igame/flutter_image_view");
        channel.setMethodCallHandler(new TipFlutterImageViewPlugin(registrar.textures(), registrar.context()));
    }

    @Override
    public void onMethodCall(MethodCall call, Result result) {
        Map arguments = (Map) call.arguments;
        if (call.method.equals("create")) {
            TextureRegistry.SurfaceTextureEntry entry = textures.createSurfaceTexture();
            SurfaceTexture surfaceTexture = entry.surfaceTexture();
            String url = arguments.get("url").toString();
            int width = ((Double) arguments.get("width")).intValue();
            int height = ((Double) arguments.get("height")).intValue();
            double aspectRatio = (double) arguments.get("aspectRatio");
            boolean centerCrop = (boolean) arguments.get("centerCrop");
            surfaceTexture.setDefaultBufferSize((int) (width * aspectRatio), (int) (height * aspectRatio));
            ImageRenderWorker worker = new ImageRenderWorker(this.context, (int) entry.id(), url, (int) (width * aspectRatio), (int) (height * aspectRatio), centerCrop);
            OpenGLRenderer render = new OpenGLRenderer(surfaceTexture, worker);
            renders.put(entry.id(), render);
            result.success(entry.id());
        } else if (call.method.equals("dispose")) {
            long textureId = ((Number) arguments.get("textureId")).longValue();
            OpenGLRenderer render = renders.get(textureId);
            if (render != null) {
                render.onDispose();
            }
            renders.delete(textureId);
        } else {
            result.notImplemented();
        }
    }
}
