import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

typedef void FlutterImageViewCreatedCallback(ImageViewController controller);

class TipFlutterImageView extends StatefulWidget {
  const TipFlutterImageView(
      {Key key,
      this.url,
      this.width,
      this.height,
      this.aspectRatio,
      this.centerCrop = true})
      : super(key: key);
  final String url;
  final double width;
  final double height;
  final double aspectRatio;
  final bool centerCrop; //是否剧中裁剪

  @override
  State<StatefulWidget> createState() => _TipFlutterImageViewState();
}

class _TipFlutterImageViewState extends State<TipFlutterImageView> {
  ImageViewController _controller;
  int textureId = -1;

  @override
  Widget build(BuildContext context) {
    return Texture(
      textureId: textureId,
    );
  }

  @override
  void initState() {
    super.initState();
    _controller = new ImageViewController.withUrl(widget.url, (_controller) {
      setState(() {
        textureId = _controller.textureId;
      });
    });
    _controller.initialize(
        width: widget.width,
        height: widget.height,
        aspectRatio: widget.aspectRatio,
        centerCrop: widget.centerCrop);
  }

  @override
  void dispose() {
    super.dispose();
    //不加上这个会内存泄漏
    _controller.dispose();

    setState(() {
      textureId = -1;
    });
  }
}

class ImageViewController {
  int textureId;
  MethodChannel _channel;
  String url;
  FlutterImageViewCreatedCallback onImageViewCreated;

  ImageViewController.withUrl(
      String url, FlutterImageViewCreatedCallback onImageViewCreated) {
    _channel = new MethodChannel('com.tencent.igame/flutter_image_view');
    this.url = url;
    this.onImageViewCreated = onImageViewCreated;
  }

  Future initialize(
      {double width,
      double height,
      double aspectRatio,
      bool centerCrop = true}) async {
    textureId = await _channel.invokeMethod('create', {
      'width': width,
      'height': height,
      'url': this.url,
      'aspectRatio': aspectRatio,
      'centerCrop': centerCrop
    });
    this.onImageViewCreated(this);
  }

  Future<Null> dispose() =>
      _channel.invokeMethod('dispose', {'textureId': textureId});

  bool get isInitialized => textureId != null;
}
